# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: lxdbapi/responses.proto
"""Generated protocol buffer code."""
from ._vendor.google.protobuf import descriptor as _descriptor
from ._vendor.google.protobuf import descriptor_pool as _descriptor_pool
from ._vendor.google.protobuf import message as _message
from ._vendor.google.protobuf import reflection as _reflection
from ._vendor.google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from lxdbapi import common_pb2 as lxdbapi_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17lxdbapi/responses.proto\x12\x07lxdbapi\x1a\x14lxdbapi/common.proto\"\xe1\x01\n\x11ResultSetResponse\x12\x15\n\rconnection_id\x18\x01 \x01(\t\x12\x14\n\x0cstatement_id\x18\x02 \x01(\r\x12\x15\n\rown_statement\x18\x03 \x01(\x08\x12%\n\tsignature\x18\x04 \x01(\x0b\x32\x12.lxdbapi.Signature\x12#\n\x0b\x66irst_frame\x18\x05 \x01(\x0b\x32\x0e.lxdbapi.Frame\x12\x14\n\x0cupdate_count\x18\x06 \x01(\x04\x12&\n\x08metadata\x18\x07 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"\x81\x01\n\x0f\x45xecuteResponse\x12+\n\x07results\x18\x01 \x03(\x0b\x32\x1a.lxdbapi.ResultSetResponse\x12\x19\n\x11missing_statement\x18\x02 \x01(\x08\x12&\n\x08metadata\x18\x03 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"f\n\x0fPrepareResponse\x12+\n\tstatement\x18\x01 \x01(\x0b\x32\x18.lxdbapi.StatementHandle\x12&\n\x08metadata\x18\x02 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"\x8a\x01\n\rFetchResponse\x12\x1d\n\x05\x66rame\x18\x01 \x01(\x0b\x32\x0e.lxdbapi.Frame\x12\x19\n\x11missing_statement\x18\x02 \x01(\x08\x12\x17\n\x0fmissing_results\x18\x03 \x01(\x08\x12&\n\x08metadata\x18\x04 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"n\n\x17\x43reateStatementResponse\x12\x15\n\rconnection_id\x18\x01 \x01(\t\x12\x14\n\x0cstatement_id\x18\x02 \x01(\r\x12&\n\x08metadata\x18\x03 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"@\n\x16\x43loseStatementResponse\x12&\n\x08metadata\x18\x01 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"@\n\x16OpenConnectionResponse\x12&\n\x08metadata\x18\x01 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"A\n\x17\x43loseConnectionResponse\x12&\n\x08metadata\x18\x01 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"s\n\x16\x43onnectionSyncResponse\x12\x31\n\nconn_props\x18\x01 \x01(\x0b\x32\x1d.lxdbapi.ConnectionProperties\x12&\n\x08metadata\x18\x02 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"\x8d\x01\n\x17\x44\x61tabasePropertyElement\x12&\n\x03key\x18\x01 \x01(\x0b\x32\x19.lxdbapi.DatabaseProperty\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.lxdbapi.TypedValue\x12&\n\x08metadata\x18\x03 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"s\n\x18\x44\x61tabasePropertyResponse\x12/\n\x05props\x18\x01 \x03(\x0b\x32 .lxdbapi.DatabasePropertyElement\x12&\n\x08metadata\x18\x02 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"\xc6\x01\n\rErrorResponse\x12\x12\n\nexceptions\x18\x01 \x03(\t\x12\x16\n\x0ehas_exceptions\x18\x07 \x01(\x08\x12\x15\n\rerror_message\x18\x02 \x01(\t\x12#\n\x08severity\x18\x03 \x01(\x0e\x32\x11.lxdbapi.Severity\x12\x12\n\nerror_code\x18\x04 \x01(\r\x12\x11\n\tsql_state\x18\x05 \x01(\t\x12&\n\x08metadata\x18\x06 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"n\n\x13SyncResultsResponse\x12\x19\n\x11missing_statement\x18\x01 \x01(\x08\x12\x14\n\x0cmore_results\x18\x02 \x01(\x08\x12&\n\x08metadata\x18\x03 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"%\n\x0bRpcMetadata\x12\x16\n\x0eserver_address\x18\x01 \x01(\t\"\x10\n\x0e\x43ommitResponse\"\x12\n\x10RollbackResponse\"\x9d\x01\n\x14\x45xecuteBatchResponse\x12\x15\n\rconnection_id\x18\x01 \x01(\t\x12\x14\n\x0cstatement_id\x18\x02 \x01(\r\x12\x15\n\rupdate_counts\x18\x03 \x03(\x04\x12\x19\n\x11missing_statement\x18\x04 \x01(\x08\x12&\n\x08metadata\x18\x05 \x01(\x0b\x32\x14.lxdbapi.RpcMetadata\"9\n\x10LxActionResponse\x12%\n\x06params\x18\x01 \x03(\x0b\x32\x15.lxdbapi.KeyValuePairB\"\n org.apache.calcite.avatica.protob\x06proto3')



_RESULTSETRESPONSE = DESCRIPTOR.message_types_by_name['ResultSetResponse']
_EXECUTERESPONSE = DESCRIPTOR.message_types_by_name['ExecuteResponse']
_PREPARERESPONSE = DESCRIPTOR.message_types_by_name['PrepareResponse']
_FETCHRESPONSE = DESCRIPTOR.message_types_by_name['FetchResponse']
_CREATESTATEMENTRESPONSE = DESCRIPTOR.message_types_by_name['CreateStatementResponse']
_CLOSESTATEMENTRESPONSE = DESCRIPTOR.message_types_by_name['CloseStatementResponse']
_OPENCONNECTIONRESPONSE = DESCRIPTOR.message_types_by_name['OpenConnectionResponse']
_CLOSECONNECTIONRESPONSE = DESCRIPTOR.message_types_by_name['CloseConnectionResponse']
_CONNECTIONSYNCRESPONSE = DESCRIPTOR.message_types_by_name['ConnectionSyncResponse']
_DATABASEPROPERTYELEMENT = DESCRIPTOR.message_types_by_name['DatabasePropertyElement']
_DATABASEPROPERTYRESPONSE = DESCRIPTOR.message_types_by_name['DatabasePropertyResponse']
_ERRORRESPONSE = DESCRIPTOR.message_types_by_name['ErrorResponse']
_SYNCRESULTSRESPONSE = DESCRIPTOR.message_types_by_name['SyncResultsResponse']
_RPCMETADATA = DESCRIPTOR.message_types_by_name['RpcMetadata']
_COMMITRESPONSE = DESCRIPTOR.message_types_by_name['CommitResponse']
_ROLLBACKRESPONSE = DESCRIPTOR.message_types_by_name['RollbackResponse']
_EXECUTEBATCHRESPONSE = DESCRIPTOR.message_types_by_name['ExecuteBatchResponse']
_LXACTIONRESPONSE = DESCRIPTOR.message_types_by_name['LxActionResponse']
ResultSetResponse = _reflection.GeneratedProtocolMessageType('ResultSetResponse', (_message.Message,), {
  'DESCRIPTOR' : _RESULTSETRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.ResultSetResponse)
  })
_sym_db.RegisterMessage(ResultSetResponse)

ExecuteResponse = _reflection.GeneratedProtocolMessageType('ExecuteResponse', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTERESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.ExecuteResponse)
  })
_sym_db.RegisterMessage(ExecuteResponse)

PrepareResponse = _reflection.GeneratedProtocolMessageType('PrepareResponse', (_message.Message,), {
  'DESCRIPTOR' : _PREPARERESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.PrepareResponse)
  })
_sym_db.RegisterMessage(PrepareResponse)

FetchResponse = _reflection.GeneratedProtocolMessageType('FetchResponse', (_message.Message,), {
  'DESCRIPTOR' : _FETCHRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.FetchResponse)
  })
_sym_db.RegisterMessage(FetchResponse)

CreateStatementResponse = _reflection.GeneratedProtocolMessageType('CreateStatementResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATESTATEMENTRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.CreateStatementResponse)
  })
_sym_db.RegisterMessage(CreateStatementResponse)

CloseStatementResponse = _reflection.GeneratedProtocolMessageType('CloseStatementResponse', (_message.Message,), {
  'DESCRIPTOR' : _CLOSESTATEMENTRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.CloseStatementResponse)
  })
_sym_db.RegisterMessage(CloseStatementResponse)

OpenConnectionResponse = _reflection.GeneratedProtocolMessageType('OpenConnectionResponse', (_message.Message,), {
  'DESCRIPTOR' : _OPENCONNECTIONRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.OpenConnectionResponse)
  })
_sym_db.RegisterMessage(OpenConnectionResponse)

CloseConnectionResponse = _reflection.GeneratedProtocolMessageType('CloseConnectionResponse', (_message.Message,), {
  'DESCRIPTOR' : _CLOSECONNECTIONRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.CloseConnectionResponse)
  })
_sym_db.RegisterMessage(CloseConnectionResponse)

ConnectionSyncResponse = _reflection.GeneratedProtocolMessageType('ConnectionSyncResponse', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTIONSYNCRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.ConnectionSyncResponse)
  })
_sym_db.RegisterMessage(ConnectionSyncResponse)

DatabasePropertyElement = _reflection.GeneratedProtocolMessageType('DatabasePropertyElement', (_message.Message,), {
  'DESCRIPTOR' : _DATABASEPROPERTYELEMENT,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.DatabasePropertyElement)
  })
_sym_db.RegisterMessage(DatabasePropertyElement)

DatabasePropertyResponse = _reflection.GeneratedProtocolMessageType('DatabasePropertyResponse', (_message.Message,), {
  'DESCRIPTOR' : _DATABASEPROPERTYRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.DatabasePropertyResponse)
  })
_sym_db.RegisterMessage(DatabasePropertyResponse)

ErrorResponse = _reflection.GeneratedProtocolMessageType('ErrorResponse', (_message.Message,), {
  'DESCRIPTOR' : _ERRORRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.ErrorResponse)
  })
_sym_db.RegisterMessage(ErrorResponse)

SyncResultsResponse = _reflection.GeneratedProtocolMessageType('SyncResultsResponse', (_message.Message,), {
  'DESCRIPTOR' : _SYNCRESULTSRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.SyncResultsResponse)
  })
_sym_db.RegisterMessage(SyncResultsResponse)

RpcMetadata = _reflection.GeneratedProtocolMessageType('RpcMetadata', (_message.Message,), {
  'DESCRIPTOR' : _RPCMETADATA,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.RpcMetadata)
  })
_sym_db.RegisterMessage(RpcMetadata)

CommitResponse = _reflection.GeneratedProtocolMessageType('CommitResponse', (_message.Message,), {
  'DESCRIPTOR' : _COMMITRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.CommitResponse)
  })
_sym_db.RegisterMessage(CommitResponse)

RollbackResponse = _reflection.GeneratedProtocolMessageType('RollbackResponse', (_message.Message,), {
  'DESCRIPTOR' : _ROLLBACKRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.RollbackResponse)
  })
_sym_db.RegisterMessage(RollbackResponse)

ExecuteBatchResponse = _reflection.GeneratedProtocolMessageType('ExecuteBatchResponse', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTEBATCHRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.ExecuteBatchResponse)
  })
_sym_db.RegisterMessage(ExecuteBatchResponse)

LxActionResponse = _reflection.GeneratedProtocolMessageType('LxActionResponse', (_message.Message,), {
  'DESCRIPTOR' : _LXACTIONRESPONSE,
  '__module__' : 'lxdbapi.responses_pb2'
  # @@protoc_insertion_point(class_scope:lxdbapi.LxActionResponse)
  })
_sym_db.RegisterMessage(LxActionResponse)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n org.apache.calcite.avatica.proto'
  _RESULTSETRESPONSE._serialized_start=59
  _RESULTSETRESPONSE._serialized_end=284
  _EXECUTERESPONSE._serialized_start=287
  _EXECUTERESPONSE._serialized_end=416
  _PREPARERESPONSE._serialized_start=418
  _PREPARERESPONSE._serialized_end=520
  _FETCHRESPONSE._serialized_start=523
  _FETCHRESPONSE._serialized_end=661
  _CREATESTATEMENTRESPONSE._serialized_start=663
  _CREATESTATEMENTRESPONSE._serialized_end=773
  _CLOSESTATEMENTRESPONSE._serialized_start=775
  _CLOSESTATEMENTRESPONSE._serialized_end=839
  _OPENCONNECTIONRESPONSE._serialized_start=841
  _OPENCONNECTIONRESPONSE._serialized_end=905
  _CLOSECONNECTIONRESPONSE._serialized_start=907
  _CLOSECONNECTIONRESPONSE._serialized_end=972
  _CONNECTIONSYNCRESPONSE._serialized_start=974
  _CONNECTIONSYNCRESPONSE._serialized_end=1089
  _DATABASEPROPERTYELEMENT._serialized_start=1092
  _DATABASEPROPERTYELEMENT._serialized_end=1233
  _DATABASEPROPERTYRESPONSE._serialized_start=1235
  _DATABASEPROPERTYRESPONSE._serialized_end=1350
  _ERRORRESPONSE._serialized_start=1353
  _ERRORRESPONSE._serialized_end=1551
  _SYNCRESULTSRESPONSE._serialized_start=1553
  _SYNCRESULTSRESPONSE._serialized_end=1663
  _RPCMETADATA._serialized_start=1665
  _RPCMETADATA._serialized_end=1702
  _COMMITRESPONSE._serialized_start=1704
  _COMMITRESPONSE._serialized_end=1720
  _ROLLBACKRESPONSE._serialized_start=1722
  _ROLLBACKRESPONSE._serialized_end=1740
  _EXECUTEBATCHRESPONSE._serialized_start=1743
  _EXECUTEBATCHRESPONSE._serialized_end=1900
  _LXACTIONRESPONSE._serialized_start=1902
  _LXACTIONRESPONSE._serialized_end=1959
# @@protoc_insertion_point(module_scope)
